<?php

namespace Drupal\webicons\Plugin\Field\FieldWidget;

use Drupal\webicons\Services\IconServiceInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\ConstraintViolationInterface;

/**
 * Defines the 'webicons_field' field widget.
 *
 * @FieldWidget(
 *   id = "webicons_field",
 *   label = @Translation("Webicons Field"),
 *   field_types = {"webicons_field"},
 * )
 */
class WebIconsFieldWidget extends WidgetBase {

  // Declare variables as "protected", otherwise in AJAX calls,
  // constructor won't be called.

  /**
   * The icon Service.
   *
   * @var IconServiceInterface
   */
  protected IconServiceInterface $iconService;

  /**
   * Constructs a new widget instance.
   *
   * @param $plugin_id
   *   The plugin id.
   * @param $plugin_definition
   *   The plugin definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition.
   * @param array $settings
   *   The settings array.
   * @param array $third_party_settings
   *   The third party settings.
   * @param \Drupal\webicons\Services\IconServiceInterface $iconService
   *   The icon service.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, IconServiceInterface $iconService) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->iconService = $iconService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('webicons.service'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $required = $delta == 0 && $this->fieldDefinition->isRequired();
    $libraryId = $items->getFieldDefinition()->getSetting('icon_library');
    $iconFactory = $this->iconService->getIconFactory($libraryId);
    $libraryName = $iconFactory->getLibraryName();
    $fieldName = $items->getFieldDefinition()->get('field_name');

    // Ensure form state contains the user input for this field
    $formValues = $form_state->getUserInput();
    $fieldValues = isset($formValues[$fieldName]) ? $formValues[$fieldName] : [];

    $wrapperId = "webiconWrapper$delta-" . $this->createUniqueId();
    $element['#prefix'] = '<div id="' . $wrapperId . '" class="webicon-wrapper">';
    $element['#suffix'] = '</div>';

    $element['fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Web icon'),
    ];

    $element['fieldset']['icon_class'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Icon class'),
      '#required' => $required,
      '#delta' => $delta,
      '#default_value' => $items[$delta]->icon_class ?? (isset($fieldValues[$delta]['fieldset']['icon_class']) ? $fieldValues[$delta]['fieldset']['icon_class'] : ''),
      '#attributes' => ['class' => ['icon-class']],
    ];

    $element['fieldset']['icon_code'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Icon code'),
      '#required' => $required,
      '#delta' => $delta,
      '#default_value' => $items[$delta]->icon_code ?? (isset($fieldValues[$delta]['fieldset']['icon_code']) ? $fieldValues[$delta]['fieldset']['icon_code'] : ''),
      '#attributes' => ['class' => ['icon-code']],
    ];

    $element['fieldset']['open-selector'] = [
      '#type' => 'html_tag',
      '#tag' => 'a',
      '#value' => $this->t('Select icon'),
      '#attributes' => [
        'href' => Url::fromRoute('webicons.open_selector', ['lid' => $libraryId, 'wid' => $wrapperId])->toString(),
        'class' => ['button', 'btn', 'use-ajax'],
        'data-dialog-type' => 'modal',
        'data-wrapperid' => $wrapperId,
      ],
    ];

    // Prepare icon values and build the preview icon component
    $keyValues = count($items[$delta]->getValue()) == 0 ? (isset($fieldValues[$delta]['fieldset']) ? $fieldValues[$delta]['fieldset'] : []) : $items[$delta]->getValue();
    if (is_null($keyValues)) {
      $keyValues = $items[$delta]->getValue();
    }

    $element['fieldset']['icon'] = $iconFactory->getIconRenderArray($keyValues, ['webicon-preview']);

    // Add required libraries
    $element['#theme_wrappers'] = ['container', 'form_element'];
    $element['#attributes']['class'][] = 'webicon-field-elements';
    $element['#attached']['library'][] = 'webicons/icon-selector';
    $element['#attached']['library'][] = $libraryName;

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function errorElement(array $element, ConstraintViolationInterface $violation, array $form, FormStateInterface $form_state) {
    return isset($violation->arrayPropertyPath[0]) ? $element[$violation->arrayPropertyPath[0]] : $element;
  }

  /**
   * This function is executed when saving field.
   * The purpose of this function is store de field values in the correct format, for example,
   * in the UI we have some datetime elements. Those elements work with DrupalDateTime. We can't store
   * the DrupalDateTime object in the database, so in this function, we must convert DrupalDateTime object
   * in the primitive type (int) and store the timestamp value.
   *
   * {@inheritdoc}
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    foreach ($values as $delta => $value) {
      if (empty($value['fieldset']['icon_class'])) {
        $values[$delta]['fieldset']['icon_class'] = NULL;
      }
      if (empty($value['fieldset']['icon_code'])) {
        $values[$delta]['fieldset']['icon_code'] = NULL;
      }
    }

    // Copy all property-values in the root structure to be saved
    foreach ($values as $delta => $value) {
      foreach ($value['fieldset'] as $k => $v) {
        $values[$delta][$k] = $v;
      }
      //@ Delete fieldset structure because all property-values were copied before
      unset($values[$delta]['fieldset']);
    }
    return $values;
  }

  /**
   * Generate random string id.
   *
   * @param $len
   *   Length of the id.
   *
   * @return string
   *   The generated id.
   */
  function createUniqueId($len = 8) {
    $validChars = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ';
    $lenChars = strlen($validChars);
    $id = '';
    for ($i = 0; $i < $len; $i++) {
      $id .= $validChars[rand(0, $lenChars - 1)];
    }
    return $id;
  }

}

