<?php

namespace Drupal\webicons\Services;

use Drupal\Core\Asset\LibraryDiscoveryInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\Config;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;

abstract class IconFactory implements IconFactoryInterface {

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   */
  protected $moduleHandler;

  /**
   * The configuration.
   *
   * @var \Drupal\Core\Config\ImmutableConfig $config
   */
  protected Config $config;

  /**
   * The Library Discovery Interface.
   *
   * @var \Drupal\Core\Asset\LibraryDiscoveryInterface $libraryDiscovery
   */
  protected LibraryDiscoveryInterface $libraryDiscovery;

  /**
   * The cache backend object.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface $cacheBackend
   */
  protected CacheBackendInterface $cacheBackend;

  /**
   * Cache id.
   */
  protected const WEBICONS_CACHE_KEY = 'webicons_collection_';

  /**
   * Contructs service object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $cfi
   *   The configuration factory object.
   * @param \Drupal\Core\Asset\LibraryDiscoveryInterface $libraryDiscovery
   *   The library discovery service.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheBackend
   *   The default cache object.
   */
  public function __construct(ModuleHandlerInterface $moduleHandler, ConfigFactoryInterface $cfi, LibraryDiscoveryInterface $libraryDiscovery, CacheBackendInterface $cacheBackend) {
    $this->moduleHandler = $moduleHandler;
    $this->config = $cfi->get('webicons.settings');
    $this->libraryDiscovery = $libraryDiscovery;
    $this->cacheBackend = $cacheBackend;
  }

  /**
   * {@inheritdoc}
   */
  public function getIcons(): array {
    $cacheIcons = $this->cacheBackend->get($this->getLibraryId());
    if ($cacheIcons) {
      return $cacheIcons->data;
    }
    return $this->extractFromLibrary();
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraryName(): string {
    return 'webicons/' . $this->getLibraryId();
  }

  /**
   * {@inheritdoc}
   */
  public function getSelectorIconsRenderTemplate(): string {
    $templatesPath = $this->moduleHandler->getModule('webicons')->getPath() . '/templates';
    return $templatesPath . '/' . 'icon-selector--items-' . $this->getLibraryId() . '.html.twig';
  }

  /**
   * {@inheritdoc}
   */
  public function getIconRenderArray(array $keyValues, array $classes=[]): array {
    return [
      '#type' => 'markup',
      '#theme' => 'webicon_field__value',
      '#item' => $keyValues,
      '#tag' => 'i',
      '#classes' => $classes,
      '#attached' => [
        'library' => [ $this->getLibraryName() ]
      ]
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraryCacheId(): string {
    return self::WEBICONS_CACHE_KEY . $this->getLibraryId();
  }

  /**
   * {@inheritdoc}
   */
  public function saveIconsIntoCache(array $icons): array {
    $this->cacheBackend->set($this->getLibraryId(), $icons, CacheBackendInterface::CACHE_PERMANENT);
    return $icons;
  }

}
